@@ -1,23 +1,23 @@
 #!/bin/bash
-# PHP-5.2	(http://www.php.net)
+# PHP-5.3	(http://www.php.net)
 #	BuildScript for Plamo Linux 4.7
-#			by Jun Kuwamura <juk@linet.gr.jp> on 2010-03-13
+#			by Jun Kuwamura <juk@linet.gr.jp> on 2009-11-22
 # require:
 HTTPD_VER=2.2.14
 HTTPD=httpd-$HTTPD_VER
 #	postgresql-8.4.x
-#	re2c-0.12.2
+#	re2c-0.13.5
 #PHP
-VERSION=5.2.13
+VERSION=5.3.1
 #VERSION=5.2.7RC4
 #SUHOSIN
 # ex. http://download.suhosin.org/suhosin-0.9.27.tgz
 SUHOSIN=suhosin-0.9.29
 # ex. http://download.suhosin.org/suhosin-patch-5.2.x-0.9.x.patch.gz
 PAT_VER=
-#PAT_VER=0.9.7
+#PAT_VER=0.9.8
 DOC_ONLY=
-#DOC_ONLY=-1	# -1: php binary package only
+DOC_ONLY=-1	# -1: php binary package only
 #DOC_ONLY=1	# 1: document package only
 
 if [ -f /var/log/packages/php ]; then
@@ -32,7 +32,7 @@ SRC_URI="http://jp.php.net/get/$SRC_FILE/from/this/mirror"
 
 if [ "$PAT_VER" != "" ]; then
   PAT_FILE=suhosin-patch-${VERSION}-${PAT_VER}.patch.gz
-  PAT_URI="http://download.suhosin.org//$PAT_FILE"
+  PAT_URI="http://download.suhosin.org/$PAT_FILE"
 fi
 #MBPAT_FILE=mbstring-jpr-25298-20050414.patch.diff.gz
 #MBPAT_URI=http://www.voltex.jp/patches/$MBPAT_FILE
@@ -100,14 +100,14 @@ export PATH=$PATH:/usr/X11R7/bin:/$PREFIX/bin
 
 if [ "$DOC_ONLY" != "1" ]; then
 
-if [ -d $PAC_NAME ]; then
- rm -f ${PAC_NAME}/ext/standard/crypt_blowfish.*
- rm -f ${PAC_NAME}/ext/standard/sha256.*
- rm -f ${PAC_NAME}/ext/varfilter/*
- rm -f ${PAC_NAME}/main/harden*
- rm -f ${PAC_NAME}/Zend/zend_canary.*
- rm -rf ${PAC_NAME}/doc-pak/
-fi
+#if [ -d $PAC_NAME ]; then
+# rm -f ${PAC_NAME}/ext/standard/crypt_blowfish.*
+# rm -f ${PAC_NAME}/ext/standard/sha256.*
+# rm -f ${PAC_NAME}/ext/varfilter/*
+# rm -f ${PAC_NAME}/main/harden*
+# rm -f ${PAC_NAME}/Zend/zend_canary.*
+# rm -rf ${PAC_NAME}/doc-pak/
+#fi
 
 if [ ! -d $PAC_NAME ]; then
  tar xvfz $SRC_DIR/$SRC_FILE
@@ -124,7 +124,8 @@ else
 fi
 
 #	if `false`; then
-./configure  --prefix=/$PREFIX --with-config-file-path=/$CONFIG \
+./configure  --prefix=/$PREFIX \
+  --with-config-file-path=/$CONFIG --with-config-file-scan-dir=/$CONFIG/php.d \
   --with-apxs2=/opt/httpd/bin/apxs --with-pear \
   --enable-zend-multibyte --enable-mbstring --enable-mbregex \
   --enable-exif --enable-soap --enable-sockets --with-zlib \
@@ -133,42 +134,47 @@ fi
   --with-pgsql  --with-pdo-pgsql=/opt/pgsql \
   --enable-sqlite-utf8
 
-#  #--with-config-file-scan-dir
-#	fi
+  #--with-config-file-scan-dir
 
 make
-cp -p /opt/httpd/conf/httpd.conf /opt/httpd/conf/httpd.conf.orig
-mv -v /opt/httpd/modules/libphp5.so /opt/httpd/modules/libphp5.so.orig
-
-make -i install
-mv -v /opt/httpd/conf/httpd.conf.orig /opt/httpd/conf/httpd.conf
+mv /opt/httpd/conf/httpd.conf /opt/httpd/conf/httpd.conf.orig
+mv /opt/httpd/modules/libphp5.so /opt/httpd/modules/libphp5.so.orig
+make install
+mv /opt/httpd/conf/httpd.conf.orig /opt/httpd/conf/httpd.conf
 
 mkdir -p $WORK/opt/htdocs
 
 export PATH=/$PREFIX/bin:$PATH
 
 rm -rf /$PREFIX/lib/php/.channels/
-/$PREFIX/bin/pear upgrade
+rm -rf /$PREFIX/lib/php/.registry/
+pear channel-update pear.php.net
+pear upgrade
 
-/$PREFIX/bin/pear install --alldeps DB
-/$PREFIX/bin/pear install --alldeps Auth
-/$PREFIX/bin/pear install --alldeps Auth_SASL
-/$PREFIX/bin/pear install --alldeps HTML_Template_IT
-#/$PREFIX/bin/pear install HTML_Common HTML_QuickForm HTML_QuickForm_Controller
-/$PREFIX/bin/pear install --alldeps HTML_QuickForm HTML_QuickForm_Controller
-#/$PREFIX/bin/pear install XML_Parser XML_Util Config
-/$PREFIX/bin/pear install --alldeps Config
-/$PREFIX/bin/pear install --alldeps MDB2
-/$PREFIX/bin/pear install --alldeps Mail_Mime
-#/$PREFIX/bin/pear install Net_Socket HTTP_Request
-/$PREFIX/bin/pear install --alldeps HTTP_Request
-/$PREFIX/bin/pear install --alldeps -f Net_URL
-/$PREFIX/bin/pear install --alldeps -f SOAP
-# /$PREFIX/bin/pear install Date Calendar
-/$PREFIX/bin/pear install --alldeps -f Calendar
+pear install --alldeps DB
+pear install --alldeps Auth
+pear install --alldeps Auth_SASL
+pear install --alldeps HTML_Template_IT
+#pear install HTML_Common HTML_QuickForm HTML_QuickForm_Controller
+pear install --alldeps HTML_QuickForm HTML_QuickForm_Controller
+#pear install XML_Parser XML_Util Config
+pear install --alldeps Config
+pear install --alldeps MDB2
+pear install MDB2_Driver_pgsql
+pear install MDB2_Driver_sqlite
+
+pear install --alldeps Mail_Mime
+#pear install Net_Socket HTTP_Request
+pear install --alldeps -f Net_URL
+pear install --alldeps HTTP_Request
+pear install --alldeps -f Net_URL2
+pear install --alldeps -f HTTP_Request2
+pear install --alldeps -f SOAP
+# pear install Date Calendar
+pear install --alldeps -f Calendar
 
 
-/$PREFIX/bin/pear upgrade-all
+pear upgrade-all
 
 ZTS=`sh ./scripts/php-config --extension-dir`
 
@@ -442,7 +448,7 @@ __EOFxmlwriter
    cd ..
 
 # APC3
- pecl download APC
+ pecl download channel://pecl.php.net/apc-3.1.3p1
  tar xvfz APC-3.*.tgz
  cd APC-3.*/
  phpize
@@ -477,7 +483,7 @@ __EOFnamazu
 
 if [ "$SUHOSIN" != "" ]; then
 #curl -o $SUHOSIN.tgz http://www.hardened-php.net/suhosin/_media/$SUHOSIN.tgz\?id=download&cache=cache
-wget http://download.suhosin.org/$SUHOSIN.tgz
+wget -N http://download.suhosin.org/$SUHOSIN.tgz
 tar xvfz $SUHOSIN.tgz
 cd $SUHOSIN/
   phpize 
@@ -546,17 +552,18 @@ cp $CUSTOM_DIR/httpd.php5-sample.conf opt/$HTTPD/conf/conf.d/
 mkdir -p opt/$HTTPD/modules
 cp -p /opt/$HTTPD/modules/libphp5.so opt/$HTTPD/modules/libphp5.so.$VERSION
 cp -p /opt/$HTTPD/modules/libphp5.so.orig /opt/$HTTPD/modules/libphp5.so
-mkdir -p opt/$HTTPD/man/man1
-cp -p /opt/$HTTPD/man/man1/*.1 opt/$HTTPD/man/man1/
-gzip opt/$HTTPD/man/man1/*.1
 
 cp -av $CUSTOM_DIR/htdocs/* $WORK/opt/htdocs/
-tar xvfz $CUSTOM_DIR/emacssl.tgz -C $WORK/usr/share
+chown -R root:root $WORK/opt/htdocs/
+tar xvfz $CUSTOM_DIR/emacssl.tgz -C usr/share
 curl -o $CUSTOM_DIR/php_browscap.ini http://browsers.garykeith.com/stream.asp\?PHP_BrowsCapINI
 cp $CUSTOM_DIR/php_browscap.ini $WORK/$PREFIX/lib/php/
+chown -R root:root $WORK/$PREFIX/
+
 
 mkdir -p $WORK/etc
 cp -a $CUSTOM_DIR/profile.d/ $WORK/etc/
+chown -R root:root $WORK/etc/
 mkdir -p $WORK/$CONFIG
 if [ "x$PAT_VER" != "x" ]; then
  sed -e "s%_ZTS_%$ZTS%" \
@@ -589,7 +596,7 @@ __EOCL__
 
 
 # Make Package
-tar cvfz ${PACKAGE}.tgz $PREFIX/*/* opt/${HTTPD}/*/* $DOC_DIR usr/share/emacs/site-lisp/*  opt/htdocs/* $CONFIG/php.ini-plamo etc/profile.d/php5.* install/doinst.sh > package.lis
+tar cvfz ${PACKAGE}.tgz $PREFIX/*/* opt/${HTTPD}/*/* $DOC_DIR usr/share/emacs/site-lisp/*  opt/htdocs/* $CONFIG/php.ini-plamo etc/profile.d/php5.* install/doinst.sh 
 mv -v ${PACKAGE}.tgz $PAC_DIR
 echo "$PAC_DIR/${PACKAGE}.tgz ... done"
 cd $CWD
